from discordmenu.discord_client import update_embed_control
from discordmenu.intra_message_state import IntraMessageState
from test.test_menu import embed_menu_1


def _get_emoji_name(raw_reaction_add_event):
    emoji_obj = raw_reaction_add_event.emoji
    if isinstance(emoji_obj, str):
        return emoji_obj
    return raw_reaction_add_event.emoji.name


async def should_respond(reaction_filters, message, event):
    for reaction_filter in reaction_filters:
        allow = await reaction_filter.allow_reaction_raw(message, event)
        if not allow:
            return False
    return True


async def test_raw_reaction_add_better(self, event):
    channel = self.bot.get_channel(event.channel_id)
    message = await channel.fetch_message(event.message_id)

    ims = IntraMessageState.extract_data(message.embeds[0])
    original_author_id = ims['original_author_id']

    embed_menu = embed_menu_1(original_author_id)

    if not (await embed_menu.should_respond(message, event)):
        return

    emoji_name = _get_emoji_name(event)

    next_embed_control = await embed_menu.next_embed_control(message, emoji_name)
    emoji_diff = embed_menu.diff_emojis(message, next_embed_control)

    await update_embed_control(message, next_embed_control, emoji_diff)

# @commands.Cog.listener('on_reaction_add')
# async def on_reaction_add(self, reaction, member):
#     from discordmenu.on_reaction import ReactionResponder
#     responder = ReactionResponder(should_respond_custom=PadInfo.should_respond_custom)
#     func_map = {
#         '🧪': PadInfo.embed_2,
#         'bcookie': PadInfo.embed_1
#     }
#
#     from discordmenu.intra_message_state import IntraMessageState
#     ims = IntraMessageState.find_data(reaction.message.embeds[0])
#     print("ims", ims)
#
#     await responder.on_reaction_add_event(reaction, member, reaction.message, func_map, ims)



from abc import abstractmethod
from collections.abc import Mapping
from typing import List


class Box:
    def __init__(self, *args, delimiter="\n"):
        self._inner_object = [a for a in args if a]
        self.delimiter = delimiter

    def _get_markdown(self, arg):
        if hasattr(arg, 'to_markdown'):
            return arg.to_markdown()
        return arg

    def to_markdown(self) -> str:
        if len(self._inner_object) == 1:
            return self._get_markdown(self._inner_object[0])
        return self.delimiter.join([self._get_markdown(a) for a in self._inner_object])


class CustomMapping(Mapping):
    def __len__(self) -> int:
        return len(self.fields)

    def __iter__(self):
        for f in self.fields:
            yield f

    def __getitem__(self, key):
        ret = getattr(self, key)
        if hasattr(ret, 'to_markdown'):
            return ret.to_markdown()
        return ret

    @property
    @abstractmethod
    def fields(self) -> List[str]:
        pass







import asyncio
from typing import Dict, List, Optional

from discord import Embed, Emoji, Forbidden, Message

from discordmenu.embed.menu import EmbedControl
from discordmenu.emoji_cache import emoji_cache


async def update_message(message: Message, updated_messaged_contents, guild_message: bool,
                         emoji_diff: Dict[str, List[Emoji]] = None):
    if isinstance(updated_messaged_contents, Embed):
        await message.edit(embed=updated_messaged_contents)
    else:
        await message.edit(content=updated_messaged_contents)
    if emoji_diff:
        add = [message.add_reaction(e) for e in emoji_diff.get('add', [])]
        await asyncio.gather(*add)

        if guild_message:
            remove = [message.clear_reaction(e) for e in emoji_diff.get('remove', [])]
            await asyncio.gather(*remove)


async def remove_reaction(message: Message, emoji, member, guild_message: bool):
    if not guild_message:
        # bots don't have permission to delete reactions in DM. So check if theres a guild associated before attempting.
        return

    try:
        await message.remove_reaction(emoji, member)
    except Forbidden:
        pass


async def send_embed_control(ctx, embed_control: EmbedControl):
    new_embed = embed_control.embed_views[0].to_embed()
    message = await ctx.send(embed=new_embed)

    emoji_to_add = [emoji_cache.get_by_name(e) for e in embed_control.emoji_buttons]
    add = [message.add_reaction(e) for e in emoji_to_add]
    await asyncio.gather(*add)


async def update_embed_control(message: Message, next_embed_control: Optional[EmbedControl], emoji_diff: Dict):
    guild_message = bool(message.guild)

    if not next_embed_control:
        await message.delete()

    updated_message_contents = next_embed_control.embed_views[0].to_embed()
    await message.edit(embed=updated_message_contents)

    if emoji_diff:
        add = [message.add_reaction(e) for e in emoji_diff.get('add', [])]
        await asyncio.gather(*add)

        if guild_message:
            remove = [message.clear_reaction(e) for e in emoji_diff.get('remove', [])]
            await asyncio.gather(*remove)
